import IKCache from '../Prototype/IKCache';
export default class IKCache_Redis extends IKCache {
  constructor(config, transaction) {
    super(config, transaction);
    //
    this.awaitingConnectionQueue = null;
    //
    const localConsole = (transaction ? transaction.logger : console);
    if (config && this.tableName) { localConsole.debug(`Using cache: REDIS on host: ${this.config.host}`); }
  }
  /* Public */
  async get(key) {
    await this._connect();
    this.connection.get(key);
  }
  async set(key, value) {
    await this._connect();
    this.connection.set(key, value);
  }
  async del(key) {
    await this._connect();
    this.connection.del(key);
  }
  async mget() {
    await this._connect();
    this.connection.mget(...arguments);
  }
  async mset() {
    await this._connect();
    this.connection.mset(...arguments);
  }
  async mdel() {
    await this._connect();
    this.connection.mdel(...arguments);
  }
  /* Private */
  _isRedisClientAvailable() {
    let response = true;
    try {
      const test = require('pg')
    } catch (e) { response = false; }
    return response;
  }
  async _connect() {
    //No connection, but waiting connection queue is valid? wait until other promise
    //fulfill this promire
    if (!this.connection && this.awaitingConnectionQueue) {
      return new Promise((resolve) => this.awaitingConnectionQueue.push(resolve));
    } else if (this.connection) return this.connection; //already connected

    //Check if redis client is available
    if (!this._isRedisClientAvailable()) throw new Error('Redis client is not available! Please, run "npm i -S redis".');

    //Prepare
    const localConsole = (this.transaction ? this.transaction.logger : console);
    this.awaitingConnectionQueue = [];
    localConsole.debug("Starting remote cache connection");
    
    //Instantiate client and initiate connection
    return new Promise((resolve, reject) => {
      const redis = require("redis");
      const connection = redis.createClient({
        host: this.config.host, user: this.config.user, 
        password: this.config.password
      });
      connection.on('connect', () => {
        this.connection = connection;
        this.awaitingConnectionQueue.forEach((resolve) => resolve());
        this.awaitingConnectionQueue = null;
        resolve();
      });
      connection.on('error', (err) => {
        this.awaitingConnectionQueue.forEach((resolve) => resolve()); //fulfill, error will be thrown
        this.awaitingConnectionQueue = null;
        reject(err);
      });
    });
  }
}
